@@ -1,17 +1,20 @@
 """KayPENTAX Disordered Voice Database Reader module"""
 
-__version__ = "0.6.0"
+__version__ = "0.6.1"
 
 import pandas as pd
 from os import path
 import numpy as np
 from glob import glob
-import re, operator
+import operator
 import nspfile
-from typing import Literal, Tuple, List
+from collections.abc import Sequence
+from typing import Literal, Tuple, List, Union, Iterator, Optional
 
 TaskType = Literal["AH", "RAINBOW"]
 
+ChannelType = Union[str, int, Sequence[Union[str, int]]]
+
 DataField = Literal[
     # fmt:off
     "PAT_ID", "VISITDATE", "FILE AH", "NORM", "FILE RAINBOW", "AGE", "SEX",
@@ -86,8 +89,8 @@ class KPDVDB:
             dtype={
                 # fmt: off
                 **{col: 'Int32' for col in ["AGE", "#", "NVB", "NSH", "NUV", "SEG", "PER"]},
-                **{col: "string" for col in ["PAT_ID", "FILE VOWEL 'AH'", "DIAGNOSIS", "LOCATION"]},
-                **{col: "category" for col in ["SEX", "NATLANG", "ORIGIN"]},
+                **{col: "string" for col in ["PAT_ID", "FILE VOWEL 'AH'", "LOCATION"]},
+                **{col: "category" for col in ["SEX", "NATLANG", "ORIGIN", "DIAGNOSIS"]},
                 **{col: float for col in ["Fo", "To", "Fhi", "Flo", "STD", "PFR", "Fftr", "Fatr", "Tsam", "Jita", 
                                         "Jitt", "RAP", "PQ", "sPPQ", "vFo", "ShdB", "Shim", "APQ", "sAPQ", 
                                         "vAm", "NHR", "VTI", "SPI", "FTRI", "ATRI", "DVB", "DSH", "DUV"]},
@@ -189,16 +192,8 @@ class KPDVDB:
 
     def _get_dx_series(self):
         if self._dx is None:  # one-time operation
-            self._dx = pd.Series(
-                [
-                    self._df_dx[
-                        (self._df_dx["PAT_ID"] == id)
-                        & (self._df_dx["VISITDATE"] == date)
-                    ]["DIAGNOSIS"]
-                    .dropna()
-                    .tolist()
-                    for id, date in zip(self._df["PAT_ID"], self._df["VISITDATE"])
-                ]
+            self._dx = (
+                self._df_dx["DIAGNOSIS"].groupby("ID").apply(lambda x: x.tolist())
             )
         return self._dx
 
@@ -274,7 +269,9 @@ class KPDVDB:
 
             if fcol == "DIAGNOSES":
                 fcond = set([fcond]) if isinstance(fcond, str) else set(fcond)
-                df = df[[len(fcond & set(v)) > 0 for v in s]]
+                df = df[
+                    s.map(lambda v: isinstance(v, list) and len(fcond & set(v)) > 0)
+                ]
             else:
                 try:  # try range/multi-choices
                     if s.dtype.kind in "iufcM":  # numeric/date
@@ -288,7 +285,9 @@ class KPDVDB:
 
         # if diagnosis screening function is given, further filter the rows
         if diagnoses_filter:
-            df = df[[diagnoses_filter(v) for v in df["DIAGNOSES"]]]
+            df = df[
+                [isinstance(v, list) and diagnoses_filter(v) for v in df["DIAGNOSES"]]
+            ]
 
             if not incl_dx:
                 # if dx not requested, drop it
@@ -318,7 +317,7 @@ class KPDVDB:
         auxdata_fields: List[DataField] = None,
         diagnoses_filter: List[str] = None,
         **filters,
-    ) -> List[str] | Tuple[List[str], pd.DataFrame]:
+    ) -> Union[List[str], Tuple[List[str], pd.DataFrame]]:
         """get NSP filepaths
 
         :param task: utterance task
@@ -395,12 +394,12 @@ class KPDVDB:
     def iter_data(
         self,
         task: TaskType,
-        channels: str | int | list = None,
+        channels: ChannelType = None,
         auxdata_fields: List[DataField] = None,
         normalize: bool = True,
         diagnoses_filter: List[str] = None,
         **filters,
-    ):
+    ) -> Iterator[Tuple[str, np.array, Optional[pd.Series]]]:
         """iterate over data samples
 
         :param task: utterance task
@@ -481,19 +480,19 @@ class KPDVDB:
 
     def read_data(
         self,
-        id: int,
+        id: str,
         task: TaskType = None,
-        channels: str | int | list = None,
+        channels: ChannelType = None,
         normalize: bool = True,
     ) -> Tuple[int, np.array]:
         """read audio data of one voice task of a recording session
 
         :param id: recording id
-        :type id: int
+        :type id: str
         :param task: voice task, defaults to None
         :type task: TaskType, optional
         :param channels: audio channel, defaults to None
-        :type channels: str | int | list, optional
+        :type channels: ChannelType, optional
         :param normalize: True to normalize data between -1 and 1, defaults to True
         :type normalize: bool, optional
         :return: tuple of sampling rate in S/s and data samples numpy array
@@ -507,3 +506,6 @@ class KPDVDB:
         if normalize:
             x = x / 2.0**15
         return fs, x
+
+    def __getitem__(self, key: str) -> Tuple[str, np.array]:
+        return self.read_data(key)
